@@ -42,6 +42,7 @@ from picard.util import (
     format_time,
     LockableObject,
     pathcmp,
+    mimetype,
     )
 
 
@@ -53,6 +54,7 @@ class File(LockableObject, Item):
         File.__id_counter += 1
         return File.__id_counter
 
+    UNDEFINED = -1
     PENDING = 0
     NORMAL = 1
     CHANGED = 2
@@ -64,6 +66,7 @@ class File(LockableObject, Item):
         self.id = self.new_id()
         self.filename = filename
         self.base_filename = os.path.basename(filename)
+        self._state = File.UNDEFINED
         self.state = File.PENDING
         self.error = None
 
@@ -131,7 +134,6 @@ class File(LockableObject, Item):
     def save(self, next, thread_pool, settings):
         metadata = Metadata()
         metadata.copy(self.metadata)
-        metadata.strip_whitespace()
         self.tagger.load_queue.put((
             partial(self._save_and_rename, self.filename, metadata, settings),
             partial(self._saving_finished, next),
@@ -171,8 +173,14 @@ class File(LockableObject, Item):
             self.state = File.NORMAL
             self.filename = new_filename = result
             length = self.orig_metadata.length
+            temp_info = {}
+            for info in ('~#bitrate', '~#sample_rate', '~#channels',
+                         '~#bits_per_sample', '~format', '~extension'):
+                temp_info[info] = self.orig_metadata[info]
             self.orig_metadata.copy(self.metadata)
             self.orig_metadata.length = length
+            for k, v in temp_info.items():
+                self.orig_metadata[k] = v
             self.metadata.changed = False
         self.update()
         return self, old_filename, new_filename
@@ -188,7 +196,8 @@ class File(LockableObject, Item):
         for name in metadata.keys():
             if isinstance(metadata[name], basestring):
                 metadata[name] = sanitize_filename(metadata[name])
-        filename = ScriptParser().eval(format, metadata)
+        format = format.replace("\t", "").replace("\n", "")
+        filename = ScriptParser().eval(format, metadata, self)
         # replace incompatible characters
         if settings["windows_compatible_filenames"] or sys.platform == "win32":
             filename = replace_win32_incompat(filename)
@@ -211,10 +220,10 @@ class File(LockableObject, Item):
 
         if settings["rename_files"]:
             # expand the naming format
-            if metadata['compilation'] == '1':
-                format = settings['va_file_naming_format']
-            else:
-                format = settings['file_naming_format']
+            format = settings['file_naming_format']
+            if settings['use_va_format']:
+                if metadata['compilation'] == '1':
+                    format = settings['va_file_naming_format']
             if len(format) > 0:
                 new_filename = self._script_to_filename(format, metadata, settings)
                 if not settings['move_files']:
@@ -227,8 +236,7 @@ class File(LockableObject, Item):
                 new_filename = new_filename.replace('/.', '/_').replace('\\.', '\\_')
                 if new_filename[0] == '.':
                     new_filename = '_' + new_filename[1:]
-
-        return os.path.join(new_dirname, new_filename + ext.lower())
+        return os.path.realpath(os.path.join(new_dirname, new_filename + ext.lower()))
 
     def _rename(self, old_filename, metadata, settings):
         new_filename, ext = os.path.splitext(
@@ -269,7 +277,7 @@ class File(LockableObject, Item):
         i = 0
         for mime, data in metadata.images:
             image_filename = filename
-            ext = ".jpg" # FIXME
+            ext = mimetype.get_extension(mime, ".jpg")
             if i > 0:
                 image_filename = "%s (%d)" % (filename, i)
             i += 1
@@ -308,7 +316,7 @@ class File(LockableObject, Item):
         if from_parent and self.parent:
             self.log.debug("Removing %r from %r", self, self.parent)
             self.parent.remove_file(self)
-        self.tagger.puidmanager.update(self.metadata['musicip_puid'], self.metadata['musicbrainz_trackid'])
+        self.tagger.puidmanager.remove(self.metadata['musicip_puid'])
         self.state = File.REMOVED
 
     def move(self, parent):
@@ -388,11 +396,21 @@ class File(LockableObject, Item):
     def get_state(self):
         return self._state
 
+
+    # in order to significantly speed up performance, the number of pending
+    #  files is cached
+    num_pending_files = 0
+
     def set_state(self, state, update=False):
+        if state != self._state:
+            if state == File.PENDING:
+                File.num_pending_files += 1
+            elif self._state == File.PENDING:
+                File.num_pending_files -= 1
         self._state = state
         if update:
             self.update()
-        self.tagger.emit(QtCore.SIGNAL("file_state_changed"))
+        self.tagger.emit(QtCore.SIGNAL("file_state_changed"), File.num_pending_files)
 
     state = property(get_state, set_state)
 
@@ -483,15 +501,12 @@ class File(LockableObject, Item):
         matches.sort(reverse=True)
         self.log.debug("Track matches: %r", matches)
 
-        if lookuptype == 'puid':
-            threshold = self.config.setting['puid_lookup_threshold']
-        else:
+        if lookuptype != 'puid':
             threshold = self.config.setting['file_lookup_threshold']
-
-        if matches[0][0] < threshold:
-            self.tagger.window.set_statusbar_message(N_("No matching tracks above the threshold for file %s"), self.filename, timeout=3000)
-            self.clear_pending()
-            return
+            if matches[0][0] < threshold:
+                self.tagger.window.set_statusbar_message(N_("No matching tracks above the threshold for file %s"), self.filename, timeout=3000)
+                self.clear_pending()
+                return
         self.tagger.window.set_statusbar_message(N_("File %s identified!"), self.filename, timeout=3000)
         self.clear_pending()
 
