@@ -27,16 +27,17 @@
 #ifdef USE_OLD_FFMPEG_LOCATIONS
 #include <avcodec.h>
 #include <avformat.h>
+#include <avio.h>
 #else
 #include <libavcodec/avcodec.h>
 #include <libavformat/avformat.h>
+#include <libavformat/avio.h>
 #endif
 #include <Python.h>
 
 #ifdef _WIN32
 
 #include <string.h>
-#include <avio.h>
 #include <fcntl.h>
 #include <windows.h>
 
@@ -114,8 +115,13 @@ ufile_write(URLContext *h, unsigned char *buf, int size)
     return _write(fd, buf, size);
 }
 
+#if LIBAVFORMAT_VERSION_MAJOR >= 52
+static int64_t
+ufile_seek(URLContext *h, int64_t pos, int whence)
+#else
 static offset_t
 ufile_seek(URLContext *h, offset_t pos, int whence)
+#endif
 {
     int fd = (size_t)h->priv_data;
     return _lseek(fd, pos, whence);
@@ -271,7 +277,7 @@ decode(PyObject *self, PyObject *args)
     sample_rate = codec_context->sample_rate;
 
     buffer_size = 135 * channels * sample_rate * 2;
-    buffer = (uint8_t *)malloc(buffer_size + AVCODEC_MAX_AUDIO_FRAME_SIZE);
+    buffer = (uint8_t *)av_malloc(buffer_size + AVCODEC_MAX_AUDIO_FRAME_SIZE);
     buffer_ptr = buffer;
     memset(buffer, 0, buffer_size);
 
@@ -314,7 +320,7 @@ decode(PyObject *self, PyObject *args)
     Py_BLOCK_THREADS
 
     return Py_BuildValue("(N,i,i,i,i)",
-        PyCObject_FromVoidPtr(buffer, free),
+        PyCObject_FromVoidPtr(buffer, av_free),
         (buffer_ptr - buffer) / 2,
         sample_rate,
         channels == 2 ? 1 : 0,
