@@ -36,9 +36,15 @@ from picard.ui.filebrowser import FileBrowser
 from picard.ui.tagsfromfilenames import TagsFromFileNamesDialog
 from picard.ui.options.dialog import OptionsDialog
 from picard.ui.tageditor import TagEditor
+from picard.ui.passworddialog import PasswordDialog
 from picard.util import icontheme, webbrowser2, find_existing_path
 from picard.util.cdrom import get_cdrom_drives
+from picard.plugin import ExtensionPoint
 
+ui_init = ExtensionPoint()
+def register_ui_init (function):
+    ui_init.register(function.__module__, function)
+    
 class MainWindow(QtGui.QMainWindow):
 
     options = [
@@ -46,7 +52,7 @@ class MainWindow(QtGui.QMainWindow):
                QtCore.QVariant.toByteArray),
         Option("persist", "window_position", QtCore.QPoint(),
                QtCore.QVariant.toPoint),
-        Option("persist", "window_size", QtCore.QSize(780, 580),
+        Option("persist", "window_size", QtCore.QSize(780, 560),
                QtCore.QVariant.toSize),
         BoolOption("persist", "window_maximized", False),
         BoolOption("persist", "view_cover_art", False),
@@ -62,8 +68,12 @@ class MainWindow(QtGui.QMainWindow):
     def setupUi(self):
         self.setWindowTitle(_("MusicBrainz Picard"))
         icon = QtGui.QIcon()
-        icon.addFile(":/images/Picard16.png", QtCore.QSize(16, 16))
-        icon.addFile(":/images/Picard32.png", QtCore.QSize(32, 32))
+        icon.addFile(":/images/16x16/picard.png", QtCore.QSize(16, 16))
+        icon.addFile(":/images/24x24/picard.png", QtCore.QSize(24, 24))
+        icon.addFile(":/images/32x32/picard.png", QtCore.QSize(32, 32))
+        icon.addFile(":/images/48x48/picard.png", QtCore.QSize(48, 48))
+        icon.addFile(":/images/128x128/picard.png", QtCore.QSize(128, 128))
+        icon.addFile(":/images/256x256/picard.png", QtCore.QSize(256, 256))
         self.setWindowIcon(icon)
 
         self.create_actions()
@@ -104,6 +114,9 @@ class MainWindow(QtGui.QMainWindow):
         # FIXME: use QApplication's clipboard
         self._clipboard = []
 
+        for function in ui_init:
+            function(self)
+
         self.restoreWindowState()
 
     def closeEvent(self, event):
@@ -132,11 +145,12 @@ class MainWindow(QtGui.QMainWindow):
     def restoreWindowState(self):
         self.restoreState(self.config.persist["window_state"])
         pos = self.config.persist["window_position"]
-        if pos.x() > 0 and pos.y() > 0:
-            self.move(pos)
         size = self.config.persist["window_size"]
+        self._desktopgeo = self.tagger.desktop().screenGeometry()
+        if pos.x() > 0 and pos.y() > 0 and pos.x()+size.width() < self._desktopgeo.width() and pos.y()+size.height() < self._desktopgeo.height():
+            self.move(pos)        
         if size.width() <= 0 or size.height() <= 0:
-            size = QtCore.QSize(780, 580)
+            size = QtCore.QSize(780, 560)
         self.resize(size)
         if self.config.persist["window_maximized"]:
             self.setWindowState(QtCore.Qt.WindowMaximized)
@@ -148,13 +162,13 @@ class MainWindow(QtGui.QMainWindow):
         self.file_counts_label = QtGui.QLabel()
         self.statusBar().addPermanentWidget(self.file_counts_label)
         self.connect(self.tagger, QtCore.SIGNAL("file_state_changed"), self.update_statusbar)
-        self.update_statusbar()
+        self.update_statusbar(0)
 
-    def update_statusbar(self):
+    def update_statusbar(self, num_pending_files):
         """Updates the status bar information."""
         self.file_counts_label.setText(" Files: %d, Pending Files: %d " % (
             self.tagger.num_files(),
-            self.tagger.num_pending_files()))
+            num_pending_files))
 
     def set_statusbar_message(self, message, *args, **kwargs):
         """Set the status bar message."""
@@ -183,29 +197,26 @@ class MainWindow(QtGui.QMainWindow):
         self.connect(self.options_action, QtCore.SIGNAL("triggered()"), self.show_options)
 
         self.cut_action = QtGui.QAction(icontheme.lookup('edit-cut', icontheme.ICON_SIZE_MENU), _(u"&Cut"), self)
-        self.cut_action.setShortcut(QtGui.QKeySequence(_(u"Ctrl+X")))
+        self.cut_action.setShortcut(QtGui.QKeySequence.Cut)
         self.cut_action.setEnabled(False)
         self.connect(self.cut_action, QtCore.SIGNAL("triggered()"), self.cut)
 
         self.paste_action = QtGui.QAction(icontheme.lookup('edit-paste', icontheme.ICON_SIZE_MENU), _(u"&Paste"), self)
-        self.paste_action.setShortcut(QtGui.QKeySequence(_(u"Ctrl+V")))
+        self.paste_action.setShortcut(QtGui.QKeySequence.Paste)
         self.paste_action.setEnabled(False)
         self.connect(self.paste_action, QtCore.SIGNAL("triggered()"), self.paste)
 
         self.help_action = QtGui.QAction(_("&Help..."), self)
 
-        # Windows, KDE and Gnome HID use F1 for this. MacOSX seems to have Cmd-H as a standard
-        if sys.platform == "darwin":
-            help_shortcut = "Ctrl+?"
-        else:
-            help_shortcut = "F1"
-
-        self.help_action.setShortcut(QtGui.QKeySequence(_(help_shortcut)))
+        self.help_action.setShortcut(QtGui.QKeySequence.HelpContents)
         self.connect(self.help_action, QtCore.SIGNAL("triggered()"), self.show_help)
 
         self.about_action = QtGui.QAction(_("&About..."), self)
         self.connect(self.about_action, QtCore.SIGNAL("triggered()"), self.show_about)
 
+        self.donate_action = QtGui.QAction(_("&Donate..."), self)
+        self.connect(self.donate_action, QtCore.SIGNAL("triggered()"), self.open_donation_page)
+
         self.report_bug_action = QtGui.QAction(_("&Report a Bug..."), self)
         self.connect(self.report_bug_action, QtCore.SIGNAL("triggered()"), self.open_bug_report)
 
@@ -215,7 +226,7 @@ class MainWindow(QtGui.QMainWindow):
         self.add_files_action = QtGui.QAction(icontheme.lookup('document-open'), _(u"&Add Files..."), self)
         self.add_files_action.setStatusTip(_(u"Add files to the tagger"))
         # TR: Keyboard shortcut for "Add Files..."
-        self.add_files_action.setShortcut(QtGui.QKeySequence(_(u"Ctrl+O")))
+        self.add_files_action.setShortcut(QtGui.QKeySequence.Open)
         self.connect(self.add_files_action, QtCore.SIGNAL("triggered()"), self.add_files)
 
         self.add_directory_action = QtGui.QAction(icontheme.lookup('folder'), _(u"A&dd Folder..."), self)
@@ -228,7 +239,7 @@ class MainWindow(QtGui.QMainWindow):
         self.save_action = QtGui.QAction(icontheme.lookup('document-save'), _(u"&Save"), self)
         self.save_action.setStatusTip(_(u"Save selected files"))
         # TR: Keyboard shortcut for "Save"
-        self.save_action.setShortcut(QtGui.QKeySequence(_(u"Ctrl+S")))
+        self.save_action.setShortcut(QtGui.QKeySequence.Save)
         self.save_action.setEnabled(False)
         self.connect(self.save_action, QtCore.SIGNAL("triggered()"), self.save)
 
@@ -245,7 +256,7 @@ class MainWindow(QtGui.QMainWindow):
 
         self.remove_action = QtGui.QAction(icontheme.lookup('list-remove'), _(u"&Remove"), self)
         self.remove_action.setStatusTip(_(u"Remove selected files/albums"))
-        self.remove_action.setShortcut(QtGui.QKeySequence("Del"))
+        self.remove_action.setShortcut(QtGui.QKeySequence.Delete)
         self.remove_action.setEnabled(False)
         self.connect(self.remove_action, QtCore.SIGNAL("triggered()"), self.remove)
 
@@ -253,6 +264,7 @@ class MainWindow(QtGui.QMainWindow):
         self.show_file_browser_action.setCheckable(True)
         if self.config.persist["view_file_browser"]:
             self.show_file_browser_action.setChecked(True)
+        self.show_file_browser_action.setShortcut(QtGui.QKeySequence(_(u"Ctrl+B")))
         self.connect(self.show_file_browser_action, QtCore.SIGNAL("triggered()"), self.show_file_browser)
 
         self.show_cover_art_action = QtGui.QAction(_(u"&Cover Art"), self)
@@ -322,6 +334,8 @@ class MainWindow(QtGui.QMainWindow):
         self.view_log_action = QtGui.QAction(_(u"View &Log..."), self)
         self.connect(self.view_log_action, QtCore.SIGNAL("triggered()"), self.show_log)
 
+        self.connect(self.tagger.xmlws, QtCore.SIGNAL("authentication_required"), self.show_password_dialog)
+
     def toggle_rename_files(self, checked):
         self.config.setting["rename_files"] = checked
 
@@ -382,6 +396,7 @@ class MainWindow(QtGui.QMainWindow):
         menu.addAction(self.report_bug_action)
         menu.addAction(self.view_log_action)
         menu.addSeparator()
+        menu.addAction(self.donate_action)
         menu.addAction(self.about_action)
 
     def update_toolbar_style(self):
@@ -551,6 +566,9 @@ class MainWindow(QtGui.QMainWindow):
     def open_support_forum(self):
         webbrowser2.open("http://forums.musicbrainz.org/viewforum.php?id=2")
 
+    def open_donation_page(self):
+        webbrowser2.open('http://metabrainz.org/donate/index.html')
+
     def save(self):
         """Tell the tagger to save the selected objects."""
         self.tagger.save(self.panel.selected_objects())
@@ -640,7 +658,10 @@ class MainWindow(QtGui.QMainWindow):
                 else:
                     metadata = obj.metadata
                     #Show dup zaper
-            elif isinstance(obj, (Cluster, Album)):
+            elif isinstance(obj, Cluster):
+                orig_metadata = obj.metadata
+                is_album = True
+            elif isinstance(obj, Album):
                 metadata = obj.metadata
                 is_album = True
 
@@ -667,6 +688,10 @@ class MainWindow(QtGui.QMainWindow):
         else:
             self.file_browser.hide()
 
+    def show_password_dialog(self, host, port, authenticator):
+        dialog = PasswordDialog(authenticator, host, port, parent=self)
+        dialog.exec_()
+
     def autotag(self):
         self.tagger.autotag(self.panel.selected_objects())
 
@@ -680,6 +705,6 @@ class MainWindow(QtGui.QMainWindow):
             target = self.tagger.unmatched_files
         else:
             target = selected_objects[0]
-        self.fileTreeView.drop_files(self.tagger.get_files_from_objects(self._clipboard), target)
+        self.panel.views[0].drop_files(self.tagger.get_files_from_objects(self._clipboard), target)
         self._clipboard = []
         self.paste_action.setEnabled(False)
