@@ -22,14 +22,13 @@
 Asynchronous XML web service.
 """
 
-import md5
+import hashlib
 import os.path
 import re
-import sha
 from PyQt4 import QtCore, QtNetwork, QtXml
 from picard import version_string
 from picard.util import partial
-from picard.const import PUID_SUBMIT_HOST, PUID_SUBMIT_PORT
+from picard.const import PUID_SUBMIT_HOST, PUID_SUBMIT_PORT, MAX_RATINGS_PER_REQUEST
 
 
 def _escape_lucene_query(text):
@@ -37,7 +36,7 @@ def _escape_lucene_query(text):
 
 
 def _md5(text):
-    m = md5.new()
+    m = hashlib.md5()
     m.update(text)
     return m.hexdigest()
 
@@ -92,12 +91,17 @@ class XmlHandler(QtXml.QXmlDefaultHandler):
 
 
 class XmlWebService(QtNetwork.QHttp):
+    """
+    Signals:
+      - authentication_required
+    """
 
     def __init__(self, cachedir, parent=None):
         QtNetwork.QHttp.__init__(self, parent)
         self.connect(self, QtCore.SIGNAL("requestStarted(int)"), self._start_request)
         self.connect(self, QtCore.SIGNAL("requestFinished(int, bool)"), self._finish_request)
         self.connect(self, QtCore.SIGNAL("readyRead(const QHttpResponseHeader &)"), self._read_data)
+        self.connect(self, QtCore.SIGNAL("authenticationRequired(const QString &, quint16, QAuthenticator *)"), self._authenticate)
         self._cachedir = cachedir
         self._request_handlers = {}
         self._last_request_time = None
@@ -111,7 +115,7 @@ class XmlWebService(QtNetwork.QHttp):
 
     def _make_cache_filename(self, host, port, path):
         url = "%s:%d%s" % (host, port, path)
-        filename = sha.new(url).hexdigest()
+        filename = hashlib.sha1(url).hexdigest()
         m = re.search(r"\.([a-z]{2,3})(?:\?|$)", url)
         if m:
             filename += "." + m.group(1)
@@ -143,8 +147,11 @@ class XmlWebService(QtNetwork.QHttp):
                 location = response.value("Location")
                 if location:
                     self.log.debug("Redirect => %s", location)
-                    location = QtCore.QUrl(location)
-                    self.get(location.host(), location.port(80), location.path(), handler, xml=xml, position=1)
+                    location = QtCore.QUrl.fromEncoded(str(location))
+                    path = location.path()
+                    if location.hasQuery():
+                        path += '?' + location.encodedQuery()
+                    self.get(location.host(), location.port(80), path, handler, xml=xml, position=1)
                     # don't call the handle for this request, only for the redirected one
                     handler = None
 
@@ -170,6 +177,9 @@ class XmlWebService(QtNetwork.QHttp):
             else:
                 self._run_next_task()
 
+    def _authenticate(self, host, port, authenticator):
+        self.emit(QtCore.SIGNAL("authentication_required"), host, port, authenticator)
+
     def _run_next_task(self):
         while len(self._queue) >= 1:
             try:
@@ -268,15 +278,17 @@ class XmlWebService(QtNetwork.QHttp):
         for name, value in kwargs.items():
             if name in ('limit', 'puid', 'discid'):
                 filters.append((name, value))
+            elif name == 'cdstubs':
+                filters.append((name, 'yes' if value else 'no'))
             else:
                 value = _escape_lucene_query(value).strip().lower()
                 if value:
-                    query.append('%s:(%s)' % (name, value.encode('utf-8')))
+                    query.append('%s:(%s)' % (name, value))
         if query:
             filters.append(('query', ' '.join(query)))
         params = []
         for name, value in filters:
-            value = str(QtCore.QUrl.toPercentEncoding(str(value)))
+            value = str(QtCore.QUrl.toPercentEncoding(QtCore.QString(value)))
             params.append('%s=%s' % (str(name), value))
         path = "/ws/1/%s/?type=xml&%s" % (entitytype, "&".join(params))
         self.get(host, port, path, handler)
@@ -305,6 +317,31 @@ class XmlWebService(QtNetwork.QHttp):
         func = partial(self._submit_puids, puids, handler)
         self.add_task(func)
 
+    def submit_ratings(self, ratings, handler):
+        """
+        Submit entity ratings to the MB server.
+        Ratings is a hash containing the numerical ratings for each
+        entity. The key of the hash is a tuple consisting of the entity type
+        and an entity ID.
+        """
+        data_list = []
+        number = 0
+        for (entitytype, entityid), rating in ratings.items():
+            data_list.append('&entity.%i=%s&id.%i=%s&rating.%i=%i' % (number, entitytype, 
+                                                                      number, entityid,
+                                                                      number, rating))
+            number = (number + 1) % MAX_RATINGS_PER_REQUEST
+        
+        self.setUser(self.config.setting["username"],
+                     self.config.setting["password"])
+        
+        i = 0
+        while i < len(data_list):
+            data = "".join(data_list[i : i + MAX_RATINGS_PER_REQUEST])
+            data = data.encode('ascii', 'ignore')
+            self.post(self.config.setting['server_host'], self.config.setting['server_port'], '/ws/1/rating/', data, handler)
+            i += MAX_RATINGS_PER_REQUEST
+
     def query_musicdns(self, handler, **kwargs):
         host = 'ofa.musicdns.org'
         port = 80
